Add missing osl() wrapper calls.
[paraslash.git] / blob.c
diff --git a/blob.c b/blob.c
index f0133c3b5d53b7c916f1662b158ac5073a74af8f..2f17502143521db2b5178ea457e3149abb60a82f 100644 (file)
--- a/blob.c
+++ b/blob.c
@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2007-2009 Andre Noll <maan@systemlinux.org>
+ * Copyright (C) 2007-2013 Andre Noll <maan@systemlinux.org>
  *
  * Licensed under the GPL v2. For licencing details see COPYING.
  */
@@ -8,7 +8,6 @@
 
 #include <regex.h>
 #include <fnmatch.h>
-#include <openssl/rc4.h>
 #include <osl.h>
 
 #include "para.h"
 #include "string.h"
 #include "afh.h"
 #include "afs.h"
-#include "net.h"
 #include "ipc.h"
 #include "portable_io.h"
+#include "sideband.h"
+#include "command.h"
 
 /**
  * Compare two osl objects pointing to unsigned integers of 32 bit size.
@@ -82,12 +82,13 @@ static struct osl_column_description blob_cols[] = {
        DEFINE_BLOB_TABLE_DESC(table_name); \
        DEFINE_BLOB_TABLE_PTR(table_name);
 
-/** \cond doxygen isn't smart enough to recognize these */
+/* doxygen isn't smart enough to recognize these */
+/** \cond blob_table */
 INIT_BLOB_TABLE(lyrics);
 INIT_BLOB_TABLE(images);
 INIT_BLOB_TABLE(moods);
 INIT_BLOB_TABLE(playlists);
-/** \endcond */
+/** \endcond blob_table */
 
 /** Flags that may be passed to the \p ls functions of each blob  type. */
 enum blob_ls_flags {
@@ -132,9 +133,12 @@ static void com_lsblob_callback(struct osl_table *table,
        struct lsblob_action_data lbad = {
                .flags = *(uint32_t *)query->data,
                .pb = {
-                       .max_size = SHMMAX,
-                       .private_data = &fd,
-                       .max_size_handler = pass_buffer_as_shm
+                       .max_size = shm_get_shmmax(),
+                       .private_data = &(struct afs_max_size_handler_data) {
+                               .fd = fd,
+                               .band = SBD_OUTPUT
+                       },
+                       .max_size_handler = afs_max_size_handler,
                }
        };
        struct pattern_match_data pmd = {
@@ -157,19 +161,21 @@ static void com_lsblob_callback(struct osl_table *table,
        ret = for_each_matching_row(&pmd);
        if (ret < 0)
                para_printf(&lbad.pb, "%s\n", para_strerror(-ret));
+       else if (pmd.num_matches == 0 && pmd.patterns.size > 0)
+               para_printf(&lbad.pb, "no matches\n");
        if (lbad.pb.offset)
-               pass_buffer_as_shm(lbad.pb.buf, lbad.pb.offset, &fd);
+               pass_buffer_as_shm(fd, SBD_OUTPUT, lbad.pb.buf, lbad.pb.offset);
        free(lbad.pb.buf);
 }
 
-static int com_lsblob(callback_function *f, struct rc4_context *rc4c, int argc, char * const * const argv)
+static int com_lsblob(callback_function *f, struct command_context *cc)
 {
        uint32_t flags = 0;
        struct osl_object options = {.data = &flags, .size = sizeof(flags)};
        int i;
 
-       for (i = 1; i < argc; i++) {
-               const char *arg = argv[i];
+       for (i = 1; i < cc->argc; i++) {
+               const char *arg = cc->argv[i];
                if (arg[0] != '-')
                        break;
                if (!strcmp(arg, "--")) {
@@ -192,21 +198,21 @@ static int com_lsblob(callback_function *f, struct rc4_context *rc4c, int argc,
        }
 //     if (argc > i)
 //             return -E_BLOB_SYNTAX;
-       return send_option_arg_callback_request(&options, argc - i,
-               argv + i, f, rc4_send_result, rc4c);
+       return send_option_arg_callback_request(&options, cc->argc - i,
+               cc->argv + i, f, afs_cb_result_handler, cc);
 }
 
 static int cat_blob(struct osl_table *table, struct osl_row *row,
                __a_unused const char *name, void *data)
 {
-       int ret = 0, ret2;
+       int ret = 0, ret2, fd = *(int *)data;
        struct osl_object obj;
 
        ret = osl(osl_open_disk_object(table, row, BLOBCOL_DEF, &obj));
        if (ret < 0)
-               return ret;
-       if (obj.size)
-               ret = pass_buffer_as_shm(obj.data, obj.size, data);
+               return (ret == osl(-E_OSL_EMPTY))? 0 : ret;
+       assert(obj.size > 0);
+       ret = pass_buffer_as_shm(fd, SBD_OUTPUT, obj.data, obj.size);
        ret2 = osl(osl_close_disk_object(&obj));
        return (ret < 0)? ret : ret2;
 }
@@ -224,23 +230,24 @@ static void com_catblob_callback(struct osl_table *table, int fd,
                .action = cat_blob
        };
        for_each_matching_row(&pmd);
+       if (pmd.num_matches == 0) {
+               char err_msg[] = "no matches\n";
+               pass_buffer_as_shm(fd, SBD_OUTPUT, err_msg, sizeof(err_msg));
+       }
 }
 
-static int com_catblob(callback_function *f, struct rc4_context *rc4c, int argc,
-               char * const * const argv)
+static int com_catblob(callback_function *f, struct command_context *cc)
 {
-       if (argc < 2)
+       if (cc->argc < 2)
                return -E_BLOB_SYNTAX;
-       return send_standard_callback_request(argc - 1, argv + 1, f,
-               rc4_send_result, rc4c);
+       return send_standard_callback_request(cc->argc - 1, cc->argv + 1, f,
+               afs_cb_result_handler, cc);
 }
 
 /** Used for removing rows from a blob table. */
 struct rmblob_data {
        /** Message buffer. */
        struct para_buffer pb;
-       /** Number of removed blobs. */
-       unsigned num_removed;
 };
 
 static int remove_blob(struct osl_table *table, struct osl_row *row,
@@ -252,7 +259,6 @@ static int remove_blob(struct osl_table *table, struct osl_row *row,
                para_printf(&rmbd->pb, "%s: %s\n", name, para_strerror(-ret));
                return ret;
        }
-       rmbd->num_removed++;
        return 1;
 }
 
@@ -261,11 +267,13 @@ static void com_rmblob_callback(struct osl_table *table, int fd,
 {
        int ret, ret2 = 0;
        struct rmblob_data rmbd = {
-               .num_removed = 0,
                .pb = {
-                       .max_size = SHMMAX,
-                       .private_data = &fd,
-                       .max_size_handler = pass_buffer_as_shm
+                       .max_size = shm_get_shmmax(),
+                       .private_data = &(struct afs_max_size_handler_data) {
+                               .fd = fd,
+                               .band = SBD_OUTPUT
+                       },
+                       .max_size_handler = afs_max_size_handler,
                }
        };
        struct pattern_match_data pmd = {
@@ -283,25 +291,24 @@ static void com_rmblob_callback(struct osl_table *table, int fd,
                if (ret2 < 0)
                        goto out;
        }
-       if (!rmbd.num_removed)
+       if (pmd.num_matches == 0)
                ret2 = para_printf(&rmbd.pb, "no matches, nothing removed\n");
        else {
-               ret2 = para_printf(&rmbd.pb, "removed %d blobs\n", rmbd.num_removed);
+               ret2 = para_printf(&rmbd.pb, "removed %d blobs\n", pmd.num_matches);
                afs_event(BLOB_RENAME, NULL, table);
        }
 out:
        if (ret2 >= 0 && rmbd.pb.offset)
-               pass_buffer_as_shm(rmbd.pb.buf, rmbd.pb.offset, &fd);
+               pass_buffer_as_shm(fd, SBD_OUTPUT, rmbd.pb.buf, rmbd.pb.offset);
        free(rmbd.pb.buf);
 }
 
-static int com_rmblob(callback_function *f, struct rc4_context *rc4c, int argc,
-               char * const * const argv)
+static int com_rmblob(callback_function *f, struct command_context *cc)
 {
-       if (argc < 2)
+       if (cc->argc < 2)
                return -E_MOOD_SYNTAX;
-       return send_option_arg_callback_request(NULL, argc - 1, argv + 1, f,
-               rc4_send_result, rc4c);
+       return send_option_arg_callback_request(NULL, cc->argc - 1, cc->argv + 1, f,
+               afs_cb_result_handler, cc);
 }
 
 static void com_addblob_callback(struct osl_table *table, __a_unused int fd,
@@ -377,7 +384,7 @@ out:
  * write input from fd to dynamically allocated buffer,
  * but maximal max_size byte.
  */
-static int fd2buf(struct rc4_context *rc4c, unsigned max_size, struct osl_object *obj)
+static int fd2buf(struct stream_cipher_context *scc, unsigned max_size, struct osl_object *obj)
 {
        const size_t chunk_size = 1024;
        size_t size = 2048, received = 0;
@@ -385,7 +392,7 @@ static int fd2buf(struct rc4_context *rc4c, unsigned max_size, struct osl_object
        char *buf = para_malloc(size);
 
        for (;;) {
-               ret = rc4_recv_bin_buffer(rc4c, buf + received, chunk_size);
+               ret = sc_recv_bin_buffer(scc, buf + received, chunk_size);
                if (ret <= 0)
                        break;
                received += ret;
@@ -407,7 +414,7 @@ static int fd2buf(struct rc4_context *rc4c, unsigned max_size, struct osl_object
 /*
  * Read data from a file descriptor, and send it to the afs process.
  *
* \param rc4c crypt context containing the file descriptor to read data from.
  * \param arg_obj Pointer to the arguments to \a f.
  * \param f The callback function.
  * \param max_len Don't read more than that many bytes from stdin.
@@ -417,14 +424,14 @@ static int fd2buf(struct rc4_context *rc4c, unsigned max_size, struct osl_object
  * This function is used by commands that wish to let para_server store
  * arbitrary data specified by the user (for instance the add_blob family of
  * commands). First, at most \a max_len bytes are read and decrypted from the
* file descriptor given by \a rc4c. The result is concatenated with the buffer
  * given by \a arg_obj, and the combined buffer is made available to the afs
  * process via the callback method. See \ref send_callback_request for details.
  *
  * \return Negative on errors, the return value of the underlying call to
  * send_callback_request() otherwise.
  */
-static int stdin_command(struct rc4_context *rc4c, struct osl_object *arg_obj,
+static int stdin_command(struct command_context *cc, struct osl_object *arg_obj,
                callback_function *f, unsigned max_len,
                callback_result_handler *result_handler,
                void *private_result_data)
@@ -432,10 +439,13 @@ static int stdin_command(struct rc4_context *rc4c, struct osl_object *arg_obj,
        struct osl_object query, stdin_obj;
        int ret;
 
-       ret = rc4_send_buffer(rc4c, AWAITING_DATA_MSG);
+       if (cc->use_sideband)
+               ret = send_sb(&cc->scc, NULL, 0, SBD_AWAITING_DATA, false);
+       else
+               ret = sc_send_buffer(&cc->scc, AWAITING_DATA_MSG);
        if (ret < 0)
                return ret;
-       ret = fd2buf(rc4c, max_len, &stdin_obj);
+       ret = fd2buf(&cc->scc, max_len, &stdin_obj);
        if (ret < 0)
                return ret;
        query.size = arg_obj->size + stdin_obj.size;
@@ -448,18 +458,17 @@ static int stdin_command(struct rc4_context *rc4c, struct osl_object *arg_obj,
        return ret;
 }
 
-static int com_addblob(callback_function *f, struct rc4_context *rc4c, int argc,
-               char * const * const argv)
+static int com_addblob(callback_function *f, struct command_context *cc)
 {
        struct osl_object arg_obj;
 
-       if (argc != 2)
+       if (cc->argc != 2)
                return -E_BLOB_SYNTAX;
-       if (!*argv[1]) /* empty name is reserved for the dummy row */
+       if (!*cc->argv[1]) /* empty name is reserved for the dummy row */
                return -E_BLOB_SYNTAX;
-       arg_obj.size = strlen(argv[1]) + 1;
-       arg_obj.data = (char *)argv[1];
-       return stdin_command(rc4c, &arg_obj, f, 10 * 1024 * 1024, NULL, NULL);
+       arg_obj.size = strlen(cc->argv[1]) + 1;
+       arg_obj.data = (char *)cc->argv[1];
+       return stdin_command(cc, &arg_obj, f, 10 * 1024 * 1024, NULL, NULL);
 }
 
 /* FIXME: Print output to client, not to log file */
@@ -485,13 +494,12 @@ out:
                PARA_NOTICE_LOG("%s\n", para_strerror(-ret));
 }
 
-static int com_mvblob(callback_function *f, __a_unused struct rc4_context *rc4c,
-               int argc, char * const * const argv)
+static int com_mvblob(callback_function *f, struct command_context *cc)
 {
-       if (argc != 3)
+       if (cc->argc != 3)
                return -E_MOOD_SYNTAX;
-       return send_option_arg_callback_request(NULL, argc - 1, argv + 1, f,
-               NULL, NULL);
+       return send_option_arg_callback_request(NULL, cc->argc - 1,
+               cc->argv + 1, f, NULL, NULL);
 }
 
 #define DEFINE_BLOB_COMMAND(cmd_name, table_name, cmd_prefix) \
@@ -499,9 +507,9 @@ static int com_mvblob(callback_function *f, __a_unused struct rc4_context *rc4c,
        { \
                return com_ ## cmd_name ## blob_callback(table_name ## _table, fd, query); \
        } \
-       int com_ ## cmd_name ## cmd_prefix(struct rc4_context *rc4c, int argc, char * const * const argv) \
+       int com_ ## cmd_name ## cmd_prefix(struct command_context *cc) \
        { \
-               return com_ ## cmd_name ## blob(com_ ## cmd_name ## cmd_prefix ## _callback, rc4c, argc, argv); \
+               return com_ ## cmd_name ## blob(com_ ## cmd_name ## cmd_prefix ## _callback, cc); \
        }
 
 static int blob_get_name_by_id(struct osl_table *table, uint32_t id,
@@ -599,7 +607,7 @@ static int blob_get_name_and_def_by_row(struct osl_table *table,
 
 /** Define the \p close function for this blob type. */
 #define DEFINE_BLOB_CLOSE(table_name) \
-       void table_name ## _close(void) \
+       static void table_name ## _close(void) \
        { \
                osl_close_table(table_name ## _table, OSL_MARK_CLEAN); \
                table_name ## _table = NULL; \
@@ -607,10 +615,10 @@ static int blob_get_name_and_def_by_row(struct osl_table *table,
 
 /** Define the \p create function for this blob type. */
 #define DEFINE_BLOB_CREATE(table_name) \
-       int table_name ## _create(const char *dir) \
+       static int table_name ## _create(const char *dir) \
        { \
                table_name ## _table_desc.dir = dir; \
-               return osl_create_table(&table_name ## _table_desc); \
+               return osl(osl_create_table(&table_name ## _table_desc)); \
        }
 
 static int blob_open(struct osl_table **table,
@@ -629,7 +637,7 @@ static int blob_open(struct osl_table **table,
 }
 
 #define DEFINE_BLOB_OPEN(table_name) \
-       int table_name ## _open(const char *dir) \
+       static int table_name ## _open(const char *dir) \
        { \
                return blob_open(&table_name ## _table, \
                        &table_name ## _table_desc, dir); \
@@ -664,9 +672,10 @@ static int blob_open(struct osl_table **table,
        DEFINE_GET_DEF_BY_NAME(table_name, cmd_prefix); \
        DEFINE_GET_NAME_AND_DEF_BY_ROW(table_name, cmd_prefix); \
 
-/** \cond doxygen isn't smart enough to recognize these */
+/* doxygen isn't smart enough to recognize these */
+/** \cond blob_function */
 DEFINE_BLOB_FUNCTIONS(lyrics, lyr);
 DEFINE_BLOB_FUNCTIONS(images, img);
 DEFINE_BLOB_FUNCTIONS(moods, mood);
 DEFINE_BLOB_FUNCTIONS(playlists, pl);
-/** \endcond */
+/** \endcond blob_function */
@@ -407,7 +414,7 @@ static int fd2buf(struct rc4_context *rc4c, unsigned max_size, struct osl_object
  * Read data from a file descriptor, and send it to the afs process.
  *
- * \param rc4c crypt context containing the file descriptor to read data from.
+ * \param scc crypt context containing the file descriptor to read data from.
  * \param arg_obj Pointer to the arguments to \a f.
  * \param f The callback function.
@@ -417,14 +424,14 @@ static int fd2buf(struct rc4_context *rc4c, unsigned max_size, struct osl_object
  * arbitrary data specified by the user (for instance the add_blob family of
  * commands). First, at most \a max_len bytes are read and decrypted from the
- * file descriptor given by \a rc4c. The result is concatenated with the buffer
+ * file descriptor given by \a scc. The result is concatenated with the buffer
  * given by \a arg_obj, and the combined buffer is made available to the afs
  * process via the callback method. See \ref send_callback_request for details.